/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"

/* Demo includes. */
//#include "basic_io.h"

#include <stdio.h>

/* includes and Declarations. */
#include "LPC17xx.h"
#include "type.h"
//#include "uart.h"
#include "logutil.h"
#include "printf.h"
#include "wifly.h"
#include "sleep.h"
#include "rtc.h"
#include "config.h"
//#include <string.h>

//extern volatile uint32_t UART0Count, UART3Count;
//extern volatile uint8_t UART0Buffer[BUFSIZE], UART3Buffer[BUFSIZE];

/* Used as a loop counter to create a very crude delay. */
#define mainDELAY_LOOP_COUNT		( 0xfffff )

#define mainQUEUE_LENGTH					( 1 )

/* The task functions. */
void vTask1( void *pvParameters );
void vTask2( void *pvParameters );

/* The queue used by both tasks. */
static xQueueHandle xQueue = NULL;

extern volatile uint8_t Alarm_On;
const char* welcomeMsg = "UART3 Online:\n";

/*-----------------------------------------------------------*/
uint32_t Task1Cont=0;

int main( void )
{
	//* Init the semi-hosting. */
	//printf( "\n" );

	/*Turn off all peripherals */
	LPC_SC->PCONP = 0x00000000;

	//UART_Flush(UART_WF);
	//UART_Flush(UART_DBG);

	/* Create the queue. */
	xQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( int32_t ) );

	if( xQueue != NULL )
	{

		/* Create one of the two tasks. */
		xTaskCreate(	vTask1,		/* Pointer to the function that implements the task. */
						(signed char *)"Task 1",	/* Text name for the task.  This is to facilitate debugging only. */
						350,		/* Stack depth in words. */
						NULL,		/* We are not using the task parameter. */
						1,			/* This task will run at priority 1. */
						NULL );		/* We are not using the task handle. */

		/* Create the other task in exactly the same way. */
		xTaskCreate( vTask2, (signed char *)"Task 2", 300, NULL, 2, NULL );

		/* Start the scheduler so our tasks start executing. */
		vTaskStartScheduler();
	}
	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );

	return 0;
}
/*-----------------------------------------------------------*/

void vTask1( void *pvParameters )
{
const char *pcTaskName = "Task 1 is running :";

Delay_Ms(DELAY,2000);

Print_Init(UART_DBG, BR_DBG);	/* baud rate setting to check 9600, 14400, 19200, 38400, 57600, 115200 */
Wifly_Init(UART_WF, BR_WF);	/* baud rate setting to check 9600, 14400, 19200, 38400, 57600, 115200 */

RTC_Init();
RTC_Count(10);

Delay_Ms(DELAY,2000);

Print_T_CRLF(UART3, (char *)welcomeMsg );
Print_T_CRLF(UART0, (char *)welcomeMsg );

Task1Cont++;
	Print_TN_CRLF(UART3, (char *)pcTaskName, Task1Cont);
	Print_TN_CRLF(UART0, (char *)pcTaskName, Task1Cont);
	Print_T_CRLF(UART3, (char *)"Ara anem a Dormir ... zzzzzzzz");
	Print_T_CRLF(UART0, (char *)"Ara anem a Dormir ... zzzzzzzz");
	//Wifly_Sleep(UART_WF);
	Delay_Ms(DELAY,2000);
	//Wifly_Sleep(UART_WF);
	portDISABLE_INTERRUPTS ();
	RTC_Start();
	Alarm_On = 0;
	Sleep();
	//portENABLE_INTERRUPTS ();
	//Wifly_WakeUp(UART_WF);
	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		Task1Cont++;
		//if (Alarm_On == 1 || UART_IRQ_On == 1)
		{
			Print_TN_CRLF(UART3, (char *)pcTaskName, Task1Cont);
			Print_TN_CRLF(UART0, (char *)pcTaskName, Task1Cont);
			//Wifly_Sleep(UART_WF);
			Delay_Ms(DELAY,1000);
			xQueueSend( xQueue, &Task1Cont, 0 );
			portDISABLE_INTERRUPTS ();
			RTC_Stop();
			RTC_Count(10);
			Wifly_Sleep(UART_WF);
			//Delay_Ms(DELAY,500);
			RTC_Start();
			Alarm_On = 0;
			//Sleep();
			//DeepSleep();
			PowerDown();
			//DeepPowerDown();
			portENABLE_INTERRUPTS ();
			SystemInit();
			Print_Init(UART_DBG, BR_DBG);	/* baud rate setting to check 9600, 14400, 19200, 38400, 57600, 115200 */
			Wifly_Init(UART_WF, BR_WF);	/* baud rate setting to check 9600, 14400, 19200, 38400, 57600, 115200 */
			RTC_Init();
			Print_T_CRLF(UART3, "Wake-Up");
			//Wifly_WakeUp(UART_WF);
			Delay_Ms(DELAY,2000);
			//SystemInit();
			//Print_Init(UART_DBG, BR_DBG);
			//Print_Init(UART_WF, BR_WF);
		}
	}
}
/*-----------------------------------------------------------*/

void vTask2( void *pvParameters )
{
const char *pcTaskName = "Task 2 is running :";
uint32_t Task2Cont;

	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		/* Print out the name of this task. */
		//vPrintString( pcTaskName );
		//vTaskSuspendAll();
		xQueueReceive( xQueue, &Task2Cont, portMAX_DELAY );

		Print_TN_CRLF(UART3, (char *)pcTaskName, Task2Cont);
		Print_TN_CRLF(UART0, (char *)pcTaskName, Task2Cont);
		//Delay_Ms(DELAY,1000);
		//portDISABLE_INTERRUPTS ();
		//RTC_Stop();
		//RTC_Count(10);
		//RTC_Start();
		//Alarm_On = 0;
		//Sleep();
		//portENABLE_INTERRUPTS ();
		//Print_T_CRLF(UART3, "Wake-Up");

		//xTaskResumeAll();
	}
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


