/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

#include "LPC17xx.h"
/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"

#include <stdio.h>

/* UART includes and Declarations. */
#include "logutil.h"
#include "printf.h"
#include "wifly.h"
#include "web.h"
#include "rtc.h"
#include "sleep.h"
#include "sensor.h"
#include "type.h"
#include "config.h"
//#include <string.h>

/* Used as a loop counter to create a very crude delay. */
#define mainDELAY_LOOP_COUNT		( 0xfffff )

#define mainQUEUE_LENGTH					( 1 )
/* The queue used by both tasks. */
static xQueueHandle xQueue = NULL;

/* The task functions. */
void vTask1( void *pvParameters );
void vTask2( void *pvParameters );

extern volatile uint8_t UART_IRQ;

extern volatile uint8_t Alarm_On;

extern volatile uint32_t Segons;
extern volatile LSensor Lectura;

/*-----------------------------------------------------------*/

uint32_t Task1Cont=0, Task2Cont=0;

//static char Buffer[LowSIZE];

const char* welcomeMsg = "UART3 Online:\n";
//static char Buf[BUFSIZE];

int main( void )
{
	/* Init the semi-hosting. */
	//printf( "\n" );

	/*Turn off all peripherals */
	LPC_SC->PCONP = 0x00000000;

	xQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( int32_t ) );

	/* Create one of the two tasks. */
	xTaskCreate(	vTask1,		/* Pointer to the function that implements the task. */
					(signed char *)"Task 1",	/* Text name for the task.  This is to facilitate debugging only. */
					350,		/* Stack depth in words. */
					NULL,		/* We are not using the task parameter. */
					1,			/* This task will run at priority 1. */
					NULL );		/* We are not using the task handle. */

	/* Create the other task in exactly the same way. */
	//xTaskCreate( vTask2, (signed char *)"Task 2", 300, NULL, 1, NULL );

	/* Start the scheduler so our tasks start executing. */
	vTaskStartScheduler();

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

void vTask1( void *pvParameters )
{
const char *pcTaskName = "Task 1 is running :";
char *http = "192.168.0.10";
char *rest = "/Default.aspx";

	//vTaskSuspendAll();

	Print_Init(UART_DBG, BR_DBG);
	Wifly_Init(UART_WF, BR_WF);
	Sensor_Init(UART_SEN, BR_SEN);

	RTC_Init();
	RTC_Count(15);
	Print_T_CRLF(UART_DBG, (char *)welcomeMsg );
	Print_T_CRLF(UART_DBG, (char *)"Ara anem a Dormir ... zzzzzzzz");

	//Wifly_Sleep(UART_WF);
	//Delay_Ms(DELAY,2000);
	//portDISABLE_INTERRUPTS ();
	//RTC_Start();
	//UART_IRQ=0;
	//Alarm_On = 0;
	//Sleep();
	//portENABLE_INTERRUPTS ();
	//Check_Read(UART_SEN, (char *)Buffer, (char *)",*", 6000, LowSIZE);

	//Delay_Ms(DELAY,3000);

	//Sensor_Reset(UART_SEN);
	//Wifly_Reset(UART_WF);
	//Wifly_WakeUp(UART_WF);
	//UART_IRQ=0;
	//if  (Alarm_On != 0)	Print_TN_CRLF(UART_DBG, (char *)">>>>>>>>> DESPERTA ALARM IRQ", Alarm_On);
	//if  (UART_IRQ != 0)	Print_TN_CRLF(UART_DBG, (char *)">>>>>>>>> DESPERTA UART IRQ", UART_IRQ);
	//xTaskResumeAll();
	for( ;; )
	{
		/* Print out the name of this task. */
		//vPrintString( pcTaskName );
		//vTaskSuspendAll();
		Lock_Task(SEM_B);

		Task1Cont++;
		Print_TN_CRLF(UART3, (char *)pcTaskName, Task1Cont);
		Print_TN_CRLF(UART0, (char *)pcTaskName, Task1Cont);
		//Wifly_Sleep(UART_WF);
		//Delay_Ms(DELAY,3000);
		//Print_T_CRLF(UART3, (char *)" ");
		//Wifly_Reset(UART_WF);
		//Sensor_Reset(UART_SEN);
		//Print_TN_CRLF(UART3, (char *)"Segons Anteriors =", Segons);
		//Sensor_Menu(UART_SEN);
		//Print_TN_CRLF(UART3, (char *)"Segons Posteriors =", Segons);
		//Delay_Ms(DELAY,500);
		//Sensor_Exit(UART_SEN);

		if (Web_Send(UART_WF, http, (char *)ALTER, rest) == TRUE )
		{
			Print_T_CRLF(UART_DBG, (char *)"-------------------- DADA ENVIADA CORRECTAMENT");
		}
		else
		{
			Print_T_CRLF(UART_DBG, (char *)"-------------------- FALLADA ENVIAMENT DE DADA");
		}

		//Delay_Ms(DELAY,6000);
		//Wifly_Sleep(UART_WF);
		Print_T_CRLF(UART_DBG, (char *)"Ara anem a Dormir ... zzzzzzzz");
		UART_IRQ = 0;
		portDISABLE_INTERRUPTS ();
		RTC_Stop();
		RTC_Count(15);
		RTC_Start();
		Alarm_On = 0;
		//if  (UART_IRQ != 0)	Print_TN_CRLF(UART_DBG, (char *)">>>>>>>>> FALLA UART IRQ", UART_IRQ);
		Sleep();
		Sleep();
		//DeepSleep();
		//PowerDown();
		//DeepPowerDown();
		portENABLE_INTERRUPTS ();
		//SystemInit();

		//Print_Init(UART_DBG, BR_DBG);
		//Wifly_Init(UART_WF, BR_WF);
		//Sensor_Init(UART_SEN, BR_SEN);
		//RTC_Init();
		if  (UART_IRQ == 10)
		{
			if (Wifly_Read(UART_WF, (char *)"CONFIG", 2000) == TRUE)
			{
				Sensor_Telnet(UART_SEN, UART_WF);
				Wifly_Close(UART_WF);
				Wifly_Exit(UART_WF);
			}
		}
		else	Wifly_WakeUp(UART_WF);
		Sensor_Read(UART_SEN);
		UART_IRQ=0;
		Alarm_On = 0;

		UnLock_Task(SEM_B);
		//Sensor_Format(UART_SEN, (char *)CVS);
		//Sensor_Units(UART_SEN, (char *)SIM);
		//Sensor_Rate(UART_SEN, 10);
		//Sensor_PresUnits(UART_SEN, (char *)MBAR);
		//Sensor_PresType(UART_SEN, (char *)ABS);
		//Sensor_Altitude(UART_SEN, 695);
		//Sensor_BaudRate(UART_SEN, BR_96);
		//Sensor_ZeroRain(UART_SEN);
		//Sensor_Attach(UART_SEN, (char *)YES);

		//Sensor_Telnet(UART_SEN, UART_WF);
		//Wifly_Close(UART_WF);
		//Print_TN_CRLF(UART3, (char *)"El valor Modificat Sample Rate es :", Segons);

		//Wifly_WakeUp(UART_WF);
		//xTaskResumeAll();

		/* Delay for a period. */
	}
}
/*-----------------------------------------------------------*/

void vTask2( void *pvParameters )
{
const char *pcTaskName = "Task 2 is running :";

	/* As per most tasks, this task is implemented in an infinite loop. */

	Delay_Ms(BRUTE,2000);
	Print_T_CRLF(UART3, (char *)welcomeMsg );
	Print_T_CRLF(UART0, (char *)welcomeMsg );

	for( ;; )
	{
		/* Print out the name of this task. */
		//vPrintString( pcTaskName );
		//vTaskSuspendAll();

		Task2Cont++;
		Print_TN_CRLF(UART3, (char *)pcTaskName, Task2Cont);
		Delay_Ms(BRUTE,1000);
		Print_TN_CRLF(UART0, (char *)pcTaskName, Task2Cont);

		//xTaskResumeAll();

	}
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	Print_T_CRLF(UART3, (char *)"Error de Malloc Failed");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	Print_T_CRLF(UART3, (char *)"Error de Stack Overflow");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


