/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

#include "LPC17xx.h"
/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
//#include "semphr.h"
#include "queue.h"

/* Utils Libray includes and Declarations. */
#include "type.h"
//#include "uart.h"
#include "logutil.h"
#include "printf.h"
#include "wifly.h"
#include "arpalab.h"
#include "config.h"

//#include <stdio.h>

/* The rate at which data is sent to the queue, specified in milliseconds. */
//#define mainQUEUE_SEND_FREQUENCY_MS			( 10000 / portTICK_RATE_MS )

/* The number of items the queue can hold.  This is 1 as the receive task
will remove items as they are added, meaning the send task should always find
the queue empty. */
#define mainQUEUE_LENGTH					( 1 )

/*
 * The tasks as described in the accompanying PDF application note.
 */
static void prvTask2( void *pvParameters );
static void prvTask1( void *pvParameters );

/* The queue used by both tasks. */
static xQueueHandle xQueue = NULL;

/* The Addresses used for Arp@Lab */
static char *http = "arpalab.appspot.com";
static char *rest = "/rest/stats?nid=502865&ip=79.144.249.10&app=test&data=";
static char *feed = "/rest/feed";
const char* welcomeMsg = "UART3 Online:\n";

/*-----------------------------------------------------------*/

int main( void )
{
	/* Create the queue. */
	xQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( int32_t ) );

	if( xQueue != NULL )
	{
		/* Start the two tasks as described in the accompanying application
		note. */
		xTaskCreate( prvTask2, ( signed char * ) "RX", 340, NULL, 1, NULL );
		xTaskCreate( prvTask1, ( signed char * ) "TX", 340, NULL, 1, NULL );

		/* Start the tasks running. */
		vTaskStartScheduler();
	}

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

static void prvTask1( void *pvParameters )
{
//portTickType xNextWakeTime;
int32_t web_feed;

	Print_Init(UART_DBG, BR_DBG);
	Arpa_Init(UART_WF, BR_WF);

	Delay_Ms(DELAY,1000);

	Print_T_CRLF(UART3, (char *)welcomeMsg );
	Print_T_CRLF(UART0, (char *)welcomeMsg );

	/*Wifly_CMD(UART_WF);

	Wifly_Set(UART_WF, (char*)"lites", 500);
	Delay_Ms(DELAY, 5000);
	Wifly_Set(UART_WF, (char*)"lites", 500);

	Wifly_Exit(UART_WF);*/

	/* Initialise xNextWakeTime - this only needs to be done once. */
	//xNextWakeTime = xTaskGetTickCount();

	for( ;; )
	{
		web_feed = Arpa_Read(UART_WF, http, feed);

		Print_T_CRLF(UART_DBG, (char *)"");
		Print_T_CRLF(UART_DBG, (char *)"---------------- INICI DEL CONTROL ---------------");
		Print_TN_CRLF(UART_DBG, (char *)"--- La Dada Recuperada es :", web_feed);
		Print_T_CRLF(UART_DBG, (char *)"--------------------------------------------------");

		xQueueSend( xQueue, &web_feed, 0 );
		//vTaskDelayUntil( &xNextWakeTime, mainQUEUE_SEND_FREQUENCY_MS );

		Delay_Ms(DELAY,5000);
	}
}

/*-----------------------------------------------------------*/

static void prvTask2( void *pvParameters )
{
int32_t rssi;
int32_t tuptime;
int32_t http_feed;

	for( ;; )
	{
		/* Wait until something arrives in the queue - this task will block
		indefinitely provided INCLUDE_vTaskSuspend is set to 1 in
		FreeRTOSConfig.h. */
		xQueueReceive( xQueue, &http_feed, portMAX_DELAY );

		/*  To get here something must have been received from the queue, but
		is it the expected value?  If it is, toggle the LED. */
		rssi = Wifly_RSSI(UART_WF);
		tuptime = Wifly_UpTime(UART_WF);

		Print_T_CRLF(UART_DBG, (char *)"-------------- FI DEL CONTROL --------------------");
		Print_TN_CRLF(UART_DBG, (char *)"--- La Dada Rebuda per Cua es :", http_feed);
		Print_TN_CRLF(UART_DBG, (char *)"--- La Dada modul 8 es :", (http_feed % 8));
		Print_TN_CRLF(UART_DBG, (char *)"--- El Temps iniciat el WiFly es :", tuptime);
		Print_T_CRLF(UART_DBG, (char *)" segons");

		if (Arpa_Send(UART_WF, http, rest, ((http_feed % 8)+rssi)) == TRUE )
		{
			Print_T_CRLF(UART_DBG, (char *)"--- DADA ENVIADA CORRECTAMENT");
		}
		else
		{
			Print_T_CRLF(UART_DBG, (char *)"--- FALLADA ENVIAMENT DE DADA");
		}

		Print_T_CRLF(UART_DBG, (char *)"--------------------------------------------------");
	}
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	//Print_T_CRLF(UART3, (char *)"Error de Malloc Failed");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	//Print_T_CRLF(UART3, (char *)"Error de Stack Overflow");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


