﻿/*
 * main.c
 *
 *  Created on: 15/05/2013
 *      Author: Jordi Fenés
 */

/* CMSIS v2 LPC17xx includes. */
#include "LPC17xx.h"
/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
/* Library includes and Declarations. */
//#include "logutil.h"
#include "printf.h"
#include "wifly.h"
#include "web.h"
#include "rtc.h"
#include "sleep.h"
#include "sensor.h"
//#include "wdt.h"
//#include "extint.h"
#include "type.h"
#include "config.h"
#include <string.h>
#include <stdlib.h>

#define mainQUEUE_LENGTH		( 1 )
/* The queues used by both tasks. */
static xQueueHandle xQueueLectu = NULL;
static xQueueHandle xQueueEnvia = NULL;

/* The Station especial functions. */
static void Put_Idle(void);
static void Station_Init(void);
static void Remote_Telnet(void);
static void Remote_Menu(void);
static void Remote_Exit(void);

/* The task functions. */
static void vTaskLectu( void *pvParameters );
static void vTaskEnvia( void *pvParameters );

/* The IRQ variables indicators of Interrupt success. */
extern volatile uint8_t UART_ON;
extern volatile uint8_t ALARM_ON;
//extern volatile uint8_t EINT_ON;

/* The Sensor variables for Readings. */
extern volatile uint32_t Segons;
extern volatile uint8_t NovaLec;
extern volatile LSensor Lectura;

/* The WEB local variables. */
static char *http = "192.168.0.10";
static char *page = "/Default.aspx";

/* The WiFly local variables. */
static uint8_t W_Mode = WPA1;
static char *W_Ssid = "WLAN_2E";
static char *W_Password = "xxxxxxxxxxxxxxx";
static char *W_IP = "192.168.0.80";
static char *W_Mask = "255.255.255.0";
static char *W_GW = "192.168.0.1";

/* The IDLE local variables. */
static uint8_t Idle = SLEEP;
static uint8_t WF_Idle = FALSE;

/* The local variables. */
static uint8_t RESET_ON = FALSE;
static uint8_t SYSTEM_FAIL = REINICI;
static uint16_t SYSTEM_ERR = 0;
static uint16_t ERR_LIMIT = 100;
static uint16_t ERR_WEB_LIMIT = 10;

/*-----------------------------------------------------------*/

int main( void )
{
	/*Turn off all peripherals */
	LPC_SC->PCONP = 0x00000000;

	xQueueLectu = xQueueCreate( mainQUEUE_LENGTH, sizeof( uint32_t ) );
	xQueueEnvia = xQueueCreate( mainQUEUE_LENGTH, sizeof( uint32_t ) );

	//if( xQueueTask2 != NULL)
	if( xQueueLectu != NULL && xQueueEnvia != NULL)
	{
		/* Create one of the two tasks. */
		xTaskCreate( vTaskEnvia, (signed char *)"Task 2 Envia", 350, NULL, 1, NULL );

		/* Create the other task in exactly the same way. */
		xTaskCreate( vTaskLectu, (signed char *)"Task 1 Lectura", 400, NULL, 1, NULL );

		/* Start the scheduler so our tasks start executing. */
		vTaskStartScheduler();
	}
	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

static void vTaskLectu( void *pvParameters )
{
uint32_t Read = TRUE;

	Station_Init();
	xQueueSend( xQueueEnvia, &Read, 0 );

	for( ;; )
	{
		xQueueReceive( xQueueLectu, &Read, portMAX_DELAY);

		if (Read == FALSE)	SYSTEM_ERR++;
		//WDT_Feed();
		/*  Enter in Power Control Mode selected */
		Put_Idle();

		if (UART_ON == 11 && Idle == SLEEP)		Read = Sensor_Read(UART_SEN);

		if  (UART_ON == 10)
		{
			Remote_Telnet();
			Read = Sensor_ReadRate(UART_SEN);
		}

		if (Idle == SLEEP && WF_Idle == TRUE)	Wifly_WakeUp(UART_WF);
		else if (Idle > SLEEP)
		{
			SystemInit();
			Station_Init();
			if (NovaLec == 1)	Read = TRUE;
		}

		if (RESET_ON == TRUE)	NVIC_SystemReset();
		if (SYSTEM_ERR >= ERR_LIMIT)
		{
			if (SYSTEM_FAIL == REINICI)		NVIC_SystemReset();
			else
			{
				Wifly_Sleep(UART_WF);
				/* Not needed disable in DeepPowerDown */
				//Print_End(UART_DBG);
				//Wifly_End(UART_WF);
				//Sensor_End(UART_SEN);
				//RTC_Stop();
				portDISABLE_INTERRUPTS();
				DeepPowerDown();
			}
		}
		UART_ON = 0;
		ALARM_ON = 0;
		//EINT_ON = 0;
		if (NovaLec == 1)	xQueueSend( xQueueEnvia, &Read, 0 );
		else
		{
			SYSTEM_ERR++;
			Read = Sensor_Reset(UART_SEN);
			if (NovaLec == 1)	xQueueSend( xQueueEnvia, &Read, 0 );
			else
			{
				Read = FALSE;
				Print_2T_CRLF(UART_DBG, UART_WF, (char *)"[ERR]--> FALLA LECTURA A Weather V3");
				xQueueSend( xQueueEnvia, &Read, 0 );
			}
		}
	}
}
/*-----------------------------------------------------------*/

static void vTaskEnvia( void *pvParameters )
{
uint32_t Send;
uint32_t Errors;
	for( ;; )
	{
		/*  Send information and control of errors sending  */
		xQueueReceive( xQueueEnvia, &Send, portMAX_DELAY);
		//WDT_Feed();
		if (Send == TRUE)
		{
			NovaLec = 0;
			/*  Send the Reading information to webserver  */
			Send = Web_Send(UART_WF, http, (char *)ALTER, page);
			if (Send == TRUE)
			{
				Print_T_CRLF(UART_DBG, (char *)"[INFO]--> DADES ENVIADES CORRECTAMENT");
			}
			else if ((Errors = Web_Send_Error(FALSE)) > ERR_WEB_LIMIT)
			{
				Print_TN_CRLF(UART_DBG, (char *)"[ERR]--> ERRORS ENVIAMENT ACUMULATS :", Errors);
				Print_TN_CRLF(UART_SEN, (char *)"[ERR]--> ERRORS ENVIAMENT ACUMULATS :", Errors);
				Web_Send_Error(TRUE);
			}
		}
		else
		{
			Send = TRUE;
			Print_2T_CRLF(UART_DBG, UART_WF, (char *)"[ERR]--> SENSE DADES PER ENVIAR AL WEB");
		}
		xQueueSend( xQueueLectu, &Send, 0 );
	}
}
/*-----------------------------------------------------------*/

static void Put_Idle(void)
{
	/*  Enter in Power Control Mode selected */
	if (WF_Idle == TRUE)	Wifly_Sleep(UART_WF);
	Print_T_CRLF(UART_DBG, (char *)"[INFO]--> Ara anem a dormir ....ZZZZZZZZZZZZ");
	portDISABLE_INTERRUPTS();
	RTC_Stop();
	if (Idle == SLEEP)	RTC_Start(Segons + 4);
	else	RTC_Start(Segons - 4);
	//WDT_Set(Segons * 10);
	UART_ON = 0;
	ALARM_ON = 0;
	//EINT_Start();
	switch(Idle)
	{
		case SLEEP:
			Sleep(); Sleep();
			break;
		case DEEPSLEEP:
			DeepSleep();
			break;
		case POWERDOWN:
			PowerDown();
			break;
	}
	portENABLE_INTERRUPTS();
	//EINT_Stop();
	return;
}
/*-----------------------------------------------------------*/

static void Station_Init(void)
{
	/*Initialize only peripherals used */
	Print_Init(UART_DBG, BR_DBG);
	Wifly_Init(UART_WF, BR_WF);
	Sensor_Init(UART_SEN, BR_SEN);
	RTC_Init();
	if (Idle == SLEEP)
	{
		/* Check if WiFly is configured */
		if (Wifly_CheckIP(UART_WF, W_IP) == FALSE)
		{
			/* Configure WiFly connection */
			Wifly_AutoConnect(UART_WF, W_Mode, W_Ssid, W_Password);
			Wifly_FixIP(UART_WF, W_IP, W_Mask, W_GW);
		}
		Sensor_ReadRate(UART_SEN);
		RTC_Count(Segons + 4);
	}
	//WDT_Init();
	//EINT_Init();
	return;
}
/*-----------------------------------------------------------*/
static void Remote_Telnet(void)
{
	/*  Telnet session by remote connection  */
	uint8_t bucle = 0;
	char buf[TERSIZE];
	char *string = NULL;
	char opt = '0';
	//WDT_Stop();
	Remote_Menu();
	Print_T_CRLF(UART_DBG, (char *)"[INFO]--> INICI DE CONEXIO REMOTA");
	while ( !bucle )
	{
		if (Check_Read(UART_WF, (char *)buf, (char *)"\r\n", 100, TERSIZE) == TRUE)
		{
			string = strtok(buf, (char *)"\r\n");
			if (strlen(string) == 1)
			{
				opt = string[0];
				switch(opt)
				{
				  case 'X':
				  case 'x':
					  bucle = 1;
					  break;
				  case 'R':
				  case 'r':
					  bucle = 1;
					  RESET_ON = TRUE;
					  break;
				  case '1':
					  /*  Redirect Telnet session to USB Weather V3 */
					  Sensor_Telnet(UART_SEN, UART_WF);
					  break;
				  case '2':
					  Print_2T_CRLF(UART_DBG, UART_WF, (char *)"[INFO]--> MODE SELECCIONAT: Sleeep");
					  Idle = SLEEP;		WF_Idle = FALSE;
					  break;
				  case '3':
					  Print_2T_CRLF(UART_DBG, UART_WF, (char *)"[INFO]--> MODE SELECCIONAT: DeepSleeep");
					  Idle = DEEPSLEEP;		WF_Idle = FALSE;
					  break;
				  case '4':
					  Print_2T_CRLF(UART_DBG, UART_WF, (char *)"[INFO]--> MODE SELECCIONAT: PowerDown");
					  Idle = POWERDOWN;		WF_Idle = FALSE;
					  break;
				  case '5':
					  Print_2T_CRLF(UART_DBG, UART_WF, (char *)"[INFO]--> MODE SELECCIONAT: Sleeep i WiFly Sleep");
					  Idle = SLEEP;		WF_Idle = TRUE;
					  break;
				  case '6':
					  Print_2T_CRLF(UART_DBG, UART_WF, (char *)"[INFO]--> MODE SELECCIONAT: DeepSleeep i WiFly Sleep");
					  Idle = DEEPSLEEP;		WF_Idle = TRUE;
					  break;
				  case '7':
					  Print_2T_CRLF(UART_DBG, UART_WF, (char *)"[INFO]--> MODE SELECCIONAT: PowerDown i WiFly Sleep");
					  Idle = POWERDOWN;		WF_Idle = TRUE;
					  break;
				  default:
					  Print_2T_CRLF(UART_WF, UART_WF, (char *)"- Invalid Option ------- ");
					  break;
				}
				Remote_Menu();
			}
		}
	}
	Remote_Exit();
	Print_T_CRLF(UART_DBG, (char *)"[INFO]--> FI DE CONEXIO REMOTA");
	free(buf);
	//WDT_Start(Segons * 10);
	return;
}

/*-----------------------------------------------------------*/
static void Remote_Menu(void)
{
	Print_T_CRLF(UART_WF, (char *)"\r\n**************************************************");
	Print_T_CRLF(UART_WF, (char *)"*******         MENU PRINCIPAL             *******");
	Print_T_CRLF(UART_WF, (char *)"**************************************************");
	Print_T_CRLF(UART_WF, (char *)"\r\n1. Configurar USB Weather V3");
	Print_T_CRLF(UART_WF, (char *)"2. Lectura per Sleep");
	Print_T_CRLF(UART_WF, (char *)"3. Lectura per DeepSleep -No connexio remota-");
	Print_T_CRLF(UART_WF, (char *)"4. Lectura per PowerDown -No connexio remota-");
	Print_T_CRLF(UART_WF, (char *)"5. Sleep amb WiFly apagat (No connexio remota)");
	Print_T_CRLF(UART_WF, (char *)"6. DeepSleep amb WiFly apagat (No connexio remota)");
	Print_T_CRLF(UART_WF, (char *)"7. PowerDown amb WiFly apagat (No connexio remota)");
	Print_T_CRLF(UART_WF, (char *)"R. Reinici del sistema");
	Print_T_CRLF(UART_WF, (char *)"X. Sortir i tancar connexio\r\n");
	return;
}

/*-----------------------------------------------------------*/

static void Remote_Exit(void)
{
	Print_T_CRLF(UART_WF, (char *)"\r\n**********************************************");
	Print_T_CRLF(UART_WF, (char *)"*    BYE ...  SESSIO REMOTA FINALITZADA      *");
	Print_T_CRLF(UART_WF, (char *)"**********************************************\r\n");
	const portTickType xDelayMs = (portTickType) 2000 / portTICK_RATE_MS;
	vTaskDelay(xDelayMs);
	Wifly_Close(UART_WF);
	Wifly_Exit(UART_WF);
	return;
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	Print_T_CRLF(UART_DBG, (char *)"[ERR]--> Error de Malloc Failed");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	Print_T_CRLF(UART_DBG, (char *)"[ERR]--> Error de Stack Overflow");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}
