/*
 * sensor.h
 *
 *  Created on: 15/05/2013
 *      Author: Jordi Fenés
 */

#ifndef SENSOR_H_
#define SENSOR_H_

typedef struct {
    float L_Tem;     /* Temperature value */
    float L_Hum;     /* Humidity percent value */
    float L_Dew;     /* Dewpoint degres value */
    float L_Bar;     /* Barometric Pressure value */
    float L_Luz;     /* Relative light level value */
    float L_Ws;      /* Wind speed km/h value */
    float L_Wd;      /* Wind direction degrees value */
    float L_Rain;    /* Cumulative rainfall value */
    float L_Bat;     /* Battery level Volts value */
} LSensor;

#define LEC		0
#define MENU	1

// Menu options for USB Weather V3

// Data Format
#define CVS		"1"
#define ANSI	"2"
#define LCD		"3"

// General Units
#define ENG		"1"
#define SIM		"2"

// Pressure Units
#define MBAR	"1"
#define INCH	"2"
#define PSI		"3"

// Pressure Type
#define ABS		"1"
#define REL		"2"

// Weather Meters Attached
#define YES		"1"
#define NOR		"2"

#define SENSOR_PIN_RST 	5

void Sensor_Parse(void);
void Sensor_Debug(void);
void Sensor_MenuRead(void);
uint8_t Sensor_Reset(uint8_t PortNum);
uint8_t Sensor_Init(uint8_t PortNum, uint32_t Baudrate);
uint8_t Sensor_End(uint8_t PortNum);
uint8_t Sensor_Read(uint8_t PortNum);
uint8_t Sendor_Set_Check(uint8_t PortNum, char *Comand, char *Check, uint32_t Time);
uint8_t Sensor_Menu(uint8_t PortNum);
uint8_t Sensor_Save(uint8_t PortNum);
uint8_t Sensor_Exit(uint8_t PortNum);
uint8_t Sensor_ReadRate(uint8_t PortNum);
uint8_t Sensor_Change(uint8_t PortNum, char *Option, char *Check, char *Selec);
uint8_t Sensor_Modif(uint8_t PortNum, char *Option, uint32_t Value);
uint8_t Sensor_Format(uint8_t PortNum, char *Selec);
uint8_t Sensor_Units(uint8_t PortNum, char *Selec);
uint8_t Sensor_Rate(uint8_t PortNum, uint32_t Value);
uint8_t Sensor_PresUnits(uint8_t PortNum, char *Selec);
uint8_t Sensor_PresType(uint8_t PortNum, char *Selec);
uint8_t Sensor_Altitude(uint8_t PortNum, uint32_t Value);
uint8_t Sensor_BaudRate(uint8_t PortNum, uint32_t Value);
uint8_t Sensor_ZeroRain(uint8_t PortNum);
uint8_t Sensor_Attach(uint8_t PortNum, char *Selec);
uint8_t Sensor_Telnet(uint8_t PortNum, uint8_t PortTerm);

#endif /* SENSOR_H_ */
