/*
 * arpalab.c
 *
 *  Created on: 12/05/2013
 *      Author: Jordi Fenés
 */

#include "type.h"
#include "logutil.h"
#include "wifly.h"
#include "arpalab.h"
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//static char Receive[GETSIZE];
static uint32_t Error_Send = 0, Error_Read = 0;

/*****************************************************************************
** Function name:		Arpa_Init
**
** Descriptions:		Defines the used UART, its baudrate and reset device
** 						This operation produce a few delay.
**
** parameters:			UART port ,  desired baudrate
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Arpa_Init(uint8_t PortNum, uint32_t Baudrate)
{
	uint8_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Task(SEM_R1);
	{
		ret = Wifly_Init(PortNum, Baudrate);
	}
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Arpa_End
**
** Descriptions:		Close the UART port used for WiFly device.
**
** parameters:			UART port
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Arpa_End(uint8_t PortNum)
{
	uint8_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Task(SEM_R1);
	{
		ret = Wifly_End(PortNum);
	}
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Arpa_Web_Send
**
** Descriptions:		Connect to specified Host and parameters (CGI address), and add Value to send
** 						with Mode HTTP/1.1
**
** parameters:			UART port to send, HOST to connect, sub-address in Host, Value to add
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Arpa_Send(uint8_t PortNum, char *Host, char *Param, int32_t Value)
{
	uint8_t ret = FALSE;
	char bufHost[BUFSIZE];
	char bufParam[GETSIZE];
	char **Receive = (char**)malloc(sizeof(char*));
	*Receive = NULL;
	Lock_Task(SEM_R1);
	sprintf(bufHost, "%s %s", Host, (char *)NETPORT);
	sprintf(bufParam, "GET %s%d %s\r\nhost: %s\r\n\r\n", Param, Value, (char *)WEB, Host);
	ret = Wifly_Web(PortNum, bufHost, bufParam, Receive);
	if (ret == TRUE)	Error_Send = 0;
	else		Error_Send++;
#if LOG >= LOG_INFO
	Log_TT_Print(LOG_INFO, ret, (char *)"Arpa_Send", (char *)"ENVIO DADES ARPALAB -> HTTP GET", Host);
#endif
	Wifly_Close(PortNum);
	UnLock_Task(SEM_R1);
	free(bufHost);
	free(bufParam);
	return ret;
}

/*****************************************************************************
** Function name:		Arpa_Web_Read
**
** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.1
** 						and collect received Value.
**
** parameters:			UART port to send, HOST to connect, sub-address in Host
** Returned value:		Value received or 0 if not correct
**
*****************************************************************************/
int32_t Arpa_Read(uint8_t PortNum, char *Host, char *Param)
{
	int32_t ret = 0;
	char bufHost[BUFSIZE];
	char bufParam[GETSIZE];
	char **Receive = (char**)malloc(sizeof(char*));
	*Receive = NULL;
	char *parse;
	Lock_Task(SEM_R1);
	sprintf(bufHost, "%s %s", Host, (char *)NETPORT);
	sprintf(bufParam, "GET %s %s\r\nhost: %s\r\n\r\n", Param, (char *)WEB, Host);
	if (Wifly_Web(PortNum, bufHost, bufParam, Receive) == TRUE)
	{
		strcpy(bufParam, *Receive);
		parse = strstr(bufParam, (char *)"chunked");
		if  (parse == NULL)
		{
			parse = strstr(bufParam, (char *)"private");
			parse = strtok(parse, (char *)"*CLOS*");
			parse = strtok(parse, (char *)"\r\n");
			parse = strtok(NULL, (char *)"\r\n");
		}
		else
		{
			parse = strtok(parse, (char *)"\r\n");
			parse = strtok(NULL, (char *)"\r\n");
			parse = strtok(NULL, (char *)"\r\n");
		}
		ret = atoi(parse);
		Error_Read = 0;
	}
	else	Error_Read++;
#if LOG >= LOG_INFO
	Log_TT_Print(LOG_INFO, ret, (char *)"Arpa_Send", (char *)"PETICIO ARPALAB -> HTTP GET", Host);
#endif
	Wifly_Close(PortNum);
	UnLock_Task(SEM_R1);
	free(bufHost);
	free(bufParam);
	return ret;
}

/*****************************************************************************
** Function name:		Arpa_Send_Error
**
** Descriptions:		Check the number of errors in send function and Initialize value.
**
** parameters:			Limit to check and True o false for initialize
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Arpa_Send_Error(uint32_t Limit, uint8_t Init)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	{
		if (Error_Send > Limit)	ret = TRUE;
		if(Init == TRUE)	Error_Send = 0;
	}
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Arpa_Read_Error
**
** Descriptions:		Check the number of errors in read function and Initialize value.
**
** parameters:			Limit to check and True o False for initialize
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Arpa_Read_Error(uint32_t Limit, uint8_t Init)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R1);
	{
		if (Error_Read > Limit)	ret = TRUE;
		if(Init == TRUE)	Error_Read = 0;
	}
	UnLock_Task(SEM_R1);
	return ret;
}

