/****************************************************************************
 *   $Id:: extint.c 5670 2010-11-19 01:33:16Z usb00423                      $
 *   Project: NXP LPC17xx EINT example
 *
 *   Description:
 *     This file contains EINT code example which include EINT 
 *     initialization, EINT interrupt handler, and APIs for EINT.
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/
#include "LPC17xx.h"
#include "type.h"
#include "extint.h"

volatile uint32_t EINT0_COUNT = 0;
volatile uint8_t EINT_ON = 0;

/*****************************************************************************
** Function name:		EINT0_Handler   (Modified)
**
** Descriptions:		external INT handler
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void EINT0_IRQHandler (void) 
{
  LPC_SC->EXTINT = EINT0;		/* clear interrupt */
  EINT_ON = 1;
  EINT0_COUNT++;
}

/*****************************************************************************
** Function name:		EINT_Init   (Modified)
**
** Descriptions:		Initialize external interrupt pin and install interrupt handler
**
** parameters:			None
** Returned value:		true or false, return false if the interrupt
**						handler can't be installed to the VIC table.
*****************************************************************************/
uint8_t EINT_Init( void )
{
  EINT0_COUNT = 0;
  EINT_ON = 0;
  //LPC_PINCON->PINSEL4 = 0x00100000;	/* set P2.10 as EINT0 and P2.0~7 GPIO output */

  /* set P2.10 as EINT0 GPIO output */
  LPC_PINCON->PINSEL4 |= (1 << 20);

  //LPC_GPIOINT->IO2IntEnF = 0x200;	/* Port2.10 is falling edge. */
  //LPC_GPIOINT->IO2IntEnR |= (0x1 << 10);	/* Port2.10 is rising edge. */
  LPC_GPIOINT->IO2IntEnF |= (1 << 10);	/* Port2.10 is falling edge. */
  LPC_SC->EXTMODE = EINT0_EDGE;			/* INT0 edge trigger */
  LPC_SC->EXTPOLAR |= (1 << 0);			/* INT0 is falling edge by default */

  //NVIC_EnableIRQ(EINT0_IRQn);
  return( TRUE );
}

/*****************************************************************************
** Function name:		EINT_Start	(New Creation)
**
** Descriptions:		Enable external interrupt
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void EINT_Start ( void )
{
  EINT0_COUNT = 0;
  EINT_ON = 0;
  NVIC_EnableIRQ(EINT0_IRQn);
  return;
}

/*****************************************************************************
** Function name:		EINT_Stop	(New Creation)
**
** Descriptions:		Disable external interrupt
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void EINT_Stop( void )
{
  NVIC_DisableIRQ(EINT0_IRQn);
  return;
}

/******************************************************************************
**                            End Of File
******************************************************************************/
