/*
 * sensor.c
 *
 *  Created on: 15/05/2013
 *      Author: Jordi Fenés
 */

#include "LPC17xx.h"
#include "type.h"
#include "logutil.h"
#include "printf.h"
#include "sensor.h"
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

volatile uint32_t Segons = 0;
volatile uint8_t NovaLec = 0;
volatile LSensor Lectura;

static char Bufsen[TIPSIZE];
static uint32_t Estat = LEC;

/*****************************************************************************
** Function name:		Sensor_Parse
**
** Descriptions:		Put the Read Weather Bufsen information in a global structure variable
**
** parameters:			none
** Returned value:		none
**
*****************************************************************************/
void Sensor_Parse(void)
{
	char *parse;
	Lock_Task(SEM_R2);
	parse = strtok(Bufsen, (char *)",");
	parse = strtok(NULL, (char *)",");
	Lectura.L_Tem = atof(parse);
	parse = strtok(NULL, (char *)",");
	Lectura.L_Hum = atof(parse);
	parse = strtok(NULL, (char *)",");
	Lectura.L_Dew = atof(parse);
	parse = strtok(NULL, (char *)",");
	Lectura.L_Bar = atof(parse);
	parse = strtok(NULL, (char *)",");
	Lectura.L_Luz = atof(parse);
	parse = strtok(NULL, (char *)",");
	Lectura.L_Ws = atof(parse);
	parse = strtok(NULL, (char *)",");
	if (strcmp(parse, (char *)"*\r\n") == 0)
	{
		Lectura.L_Bat = Lectura.L_Ws;
		Lectura.L_Ws = 0;
		Lectura.L_Wd = 0;
		Lectura.L_Rain = 0;
	}
	else
	{
		Lectura.L_Wd = atof(parse);
		parse = strtok(NULL, (char *)",");
		Lectura.L_Rain = atof(parse);
		parse = strtok(NULL, (char *)",");
		Lectura.L_Bat = atof(parse);
	}
	Sensor_Debug();
	if (Lectura.L_Bat != 0)	NovaLec = 1;
	UnLock_Task(SEM_R2);
	return;
}

/*****************************************************************************
** Function name:		Sensor_Debug
**
** Descriptions:		Print in UART_DBG port Read information from global structure variable
**
** parameters:			none
** Returned value:		none
**
*****************************************************************************/
void Sensor_Debug(void)
{
	//Lock_Task(SEM_R2);
	Print_TextFloat(UART_DBG, (char *)"[LECTURA]--> Temperatura =", Lectura.L_Tem);
	Print_TextFloat(UART_DBG, (char *)"   Humitat Rel. =", Lectura.L_Hum);
	Print_TF_CRLF(UART_DBG, (char *)"  DewPoint  =", Lectura.L_Dew);
	Print_TextFloat(UART_DBG, (char *)"[LECTURA]--> P.Barometre =", Lectura.L_Bar);
	Print_TextFloat(UART_DBG, (char *)"  Lluminositat =", Lectura.L_Luz);
	Print_TF_CRLF(UART_DBG, (char *)"   Vel. Vent =", Lectura.L_Ws);
	Print_TextFloat(UART_DBG, (char *)"[LECTURA]--> Direc. Vent =", Lectura.L_Wd);
	Print_TextFloat(UART_DBG, (char *)"   Pluja Acum.  =", Lectura.L_Rain);
	Print_TF_CRLF(UART_DBG, (char *)"   Bateria V =", Lectura.L_Bat);
	//UnLock_Task(SEM_R2);
	return;
}

/*****************************************************************************
** Function name:		Sensor_MenuRead
**
** Descriptions:		Put Read Bufsen Menu Sample Rate in a global Seconds variable
**
** parameters:			none
** Returned value:		none
**
*****************************************************************************/
void Sensor_MenuRead(void)
{
	char *parse;
	parse = strstr(Bufsen, (char *)"Sample rate (");
	parse = strtok(parse, (char *)"(");
	parse = strtok(NULL, (char *)")");
	Segons = atoi(parse);
	return;
}

/*****************************************************************************
** Function name:		Sensor_Reset
**
** Descriptions:		Hardware reset the USB Weather V3 device. PIN 6-39 (P0.5) of LPC1769
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
uint8_t Sensor_Reset(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	/* Enable GPIO into PCONP */
	LPC_SC->PCONP |= (1 << 15);
	// Initialize pin
	LPC_PINCON->PINSEL0 &= ( ~( 2 << 10 ) );
	LPC_GPIO0->FIODIR |= ( 1 << SENSOR_PIN_RST );
	// it made reset at USB Weather V3
	LPC_GPIO0->FIOCLR = 1 << SENSOR_PIN_RST;
	Delay_Ms(DELAY,7); // min 160us
	LPC_GPIO0->FIOSET = 1 << SENSOR_PIN_RST;
	ret = Check_Read(PortNum, (char *)Bufsen, (char *)",*", 6000, TIPSIZE);
	Sensor_Parse();
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Reset", (char *)"RESET DEL MODUL Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Init
**
** Descriptions:		Defines the used UART, its baudrate and reset device
** 						This operation produce a few delay.
**
** parameters:			UART port ,  desired baudrate
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Init(uint8_t PortNum, uint32_t Baudrate)
{
	uint8_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to USB Weather V3 */
	Lock_Task(SEM_R2);
	NovaLec = 0;
	Print_Flush(PortNum);
	Bufsen[0]='\0';
	ret = Print_Init(PortNum, Baudrate);
	if (ret == TRUE)	ret = Sensor_Reset(PortNum);
	Estat = LEC;
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Init", (char *)"INICIACIO MODUL Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_End
**
** Descriptions:		Close the UART port used for Sensor device.
**
** parameters:			UART port
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_End(uint8_t PortNum)
{
	uint8_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Task(SEM_R2);
	{
		Print_Flush(PortNum);
		ret = Print_End(PortNum);
	}
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_End", (char *)"TANCAMENT PORT MODUL Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Read
**
** Descriptions:		Read the UART Bufsen for received New Sensor Readings
**
** parameters:			UART Port
** Returned value:		TRUE when read or FALSE when Time finalize
**
*****************************************************************************/
uint8_t Sensor_Read(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Check_Read(PortNum, (char *)Bufsen, (char *)",*", 200, TIPSIZE);
	Sensor_Parse();
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Read", (char *)"LECTURA UART DEL MODUL Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Set_Check
**
** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
** 						and check response with pointer Value
**
** parameters:			UART port to send, char pointer to command, char pointer to check
** 						and time to wait response
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Set_Check(uint8_t PortNum, char *Comand, char *Check, uint32_t Time)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R2);
	Bufsen[0] = '\0';
	Print_Flush(PortNum);
	Print_Text(PortNum, Comand);
	ret = Check_Read(PortNum, (char *)Bufsen, Check, Time, TIPSIZE);
	//UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Menu
**
** Descriptions:		Send CTRL-Z (0x1A) to enter at Menu Mode of USB Weather V3
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Menu(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	if (Estat == MENU) return TRUE;
	ret = Sensor_Set_Check(PortNum, (char *)"\x1A", (char *)"S.", 6000);
	if (ret == TRUE) Estat = MENU;
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Menu", (char *)"ENTRADA MENU CONFIG Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Save
**
** Descriptions:		Send 'S' to save options changed at Menu Mode of USB Weather V3
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Save(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	if (Estat == LEC)	ret = TRUE;
	else
	{
		ret = Sensor_Set_Check(PortNum, (char *)"S", (char *)",*", 2000);
		Sensor_Parse();
		if (ret == TRUE) Estat = LEC;
	}
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Save", (char *)"SORTIR GUARDANT CANVIS A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Exit
**
** Descriptions:		Send 'E' to Exit without save changes at Menu Mode of USB Weather V3
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Exit(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	if (Estat == LEC)	ret = TRUE;
	else
	{
		ret = Sensor_Set_Check(PortNum, (char *)"X", (char *)",*", 2000);
		Sensor_Parse();
		if (ret == TRUE) Estat = LEC;
	}
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Exit", (char *)"SORTIR DESCARTANT CANVIS A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_ReadRate
**
** Descriptions:		Enter to Menu mode, Read the sample Rate, and exit without changes of USB Weather V3
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_ReadRate(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	if (Estat == LEC)	ret = Sensor_Set_Check(PortNum, (char *)"\x1A", (char *)"S.", 6000);
	if (ret == TRUE)
	{
		Sensor_MenuRead();
		ret = Sensor_Set_Check(PortNum, (char *)"X", (char *)",*", 2000);
		if (ret == TRUE)	Sensor_Parse();
	}
	if (ret == TRUE)	Estat = LEC;
#if LOG >= LOG_INFO
	Log_TN_Print(LOG_INFO, ret, (char *)"Sensor_ReadRate", (char *)"RECUPERAR SAMPLE RATE A Weather V3", Segons);
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Change
**
** Descriptions:		Send Option to change selected values of USB Weather V3
**
** parameters:			UART port to send, Menu Option, value to check submenu and choose selec
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Change(uint8_t PortNum, char *Option, char *Check, char *Selec)
{
	uint8_t ret = FALSE;
	//Lock_Task(SEM_R2);
	if (Estat == LEC)	ret = Sensor_Menu(PortNum);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Change", (char *)"ENTREM CANVI CONFIGURACIO A Weather V3");
#endif
	ret = Sensor_Set_Check(PortNum, Option, Check, 2000);
	if (ret == TRUE)	ret = Sensor_Set_Check(PortNum, Selec, (char *)"S.", 6000); Sensor_MenuRead();
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Change", (char *)"SORTIM CANVI CONFIGURACIO A Weather V3");
#endif
	if (ret == TRUE)	ret = Sensor_Save(PortNum);
	else	ret = Sensor_Exit(PortNum);
	//UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Modif
**
** Descriptions:		Send Option for enter Value parameter of USB Weather V3
**
** parameters:			UART port to send, Option of Menu and new Value to set
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Modif(uint8_t PortNum, char *Option, uint32_t Value)
{
	uint8_t ret = FALSE;
	char buf[TERSIZE];
	//Lock_Task(SEM_R2);
	if (Estat == LEC)	ret = Sensor_Menu(PortNum);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Modif", (char *)"ENTREM CANVI VALORS A Weather V3");
#endif
	ret = Sensor_Set_Check(PortNum, Option, (char *)":", 2000);
	sprintf(buf,"%d\n",Value);
	if (ret == TRUE)	ret = Sensor_Set_Check(PortNum, buf, (char *)"S.", 6000); Sensor_MenuRead();
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Modif", (char *)"SORTIM CANVI VALORS A Weather V3");
#endif
	if (ret == TRUE)	ret = Sensor_Save(PortNum);
	else	ret = Sensor_Exit(PortNum);
	//UnLock_Task(SEM_R2);
	free(buf);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Format
**
** Descriptions:		Send '1' Data Format desired for output USB Weather V3
**
** parameters:			UART port to send and Format selected
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Format(uint8_t PortNum, char *Selec)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Sensor_Change(PortNum, (char *)"1", (char *)"3.", Selec);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Format", (char *)"CANVIAT DATA FORMAT A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Units
**
** Descriptions:		Send '2' General Units desired for output USB Weather V3
**
** parameters:			UART port to send and Units selected
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Units(uint8_t PortNum, char *Selec)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Sensor_Change(PortNum, (char *)"2", (char *)"2.", Selec);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Units", (char *)"CANVIAT GENERAL UNITS A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Rate
**
** Descriptions:		Send '3' for enter the reading Sample Rates of USB Weather V3
**
** parameters:			UART port to send and new Value to set
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Rate(uint8_t PortNum, uint32_t Value)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Sensor_Modif(PortNum, (char *)"3", Value);
	if (ret == TRUE)	Segons = Value;
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Rate", (char *)"CANVIAT SAMPLE RATE A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_PresUnits
**
** Descriptions:		Send '4' Pressure Units desired for output USB Weather V3
**
** parameters:			UART port to send and Pressure Units selected
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_PresUnits(uint8_t PortNum, char *Selec)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Sensor_Change(PortNum, (char *)"4", (char *)"3.", Selec);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_PresUnits", (char *)"CANVIAT PRESSURE UNITS A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_PresType
**
** Descriptions:		Send '5' Pressure Type desired for output USB Weather V3
**
** parameters:			UART port to send and Pressure Type selected
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_PresType(uint8_t PortNum, char *Selec)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Sensor_Change(PortNum, (char *)"5", (char *)"2.", Selec);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_PresUnits", (char *)"CANVIAT PRESSURE TYPE A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Altitude
**
** Descriptions:		Send '6' for enter the Station Altitude of USB Weather V3
**
** parameters:			UART port to send and new Value to set
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Altitude(uint8_t PortNum, uint32_t Value)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Sensor_Modif(PortNum, (char *)"6", Value);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Altitude", (char *)"CANVIAT STATION ALTITUDE A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_BaudRate
**
** Descriptions:		Send '7' for enter the Baud Rate connection of USB Weather V3
**
** parameters:			UART port to send and new Value to set
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_BaudRate(uint8_t PortNum, uint32_t Value)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Sensor_Modif(PortNum, (char *)"7", Value);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_BaudRate", (char *)"CANVIAT BAUD RATE A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_ZeroRain
**
** Descriptions:		Send '8' to reset Rain values of USB Weather V3
**
** parameters:			UART port to send
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_ZeroRain(uint8_t PortNum)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	if (Estat == LEC)	ret = Sensor_Menu(PortNum);
	ret = Sensor_Set_Check(PortNum, (char *)"8", (char *)"S.", 6000);
	Sensor_MenuRead();
	if (ret == TRUE)	ret = Sensor_Save(PortNum);
	else	ret = Sensor_Exit(PortNum);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_ZeroRain", (char *)"SORTIR DESCARTANT CANVIS A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Attach
**
** Descriptions:		Send '9' yes/no attached meters for add/remove at output USB Weather V3
**
** parameters:			UART port to send and put attached Meters yes/no selected
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Attach(uint8_t PortNum, char *Selec)
{
	uint8_t ret = FALSE;
	Lock_Task(SEM_R2);
	ret = Sensor_Change(PortNum, (char *)"9", (char *)"2.", Selec);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_PresUnits", (char *)"CANVIAT ATTACHED METERS A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	return ret;
}

/*****************************************************************************
** Function name:		Sensor_Telnet
**
** Descriptions:		Remote Telnet control for USB Weather V3
**
** parameters:			UART port to send and UART port Remote Terminal
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Sensor_Telnet(uint8_t PortNum, uint8_t PortTerm)
{
	uint8_t ret = FALSE;
	uint8_t bucle = 0, pos = 0, tip = 0;
	char buf[TERSIZE];
	char *string = NULL;
	char opt = '0';
	Lock_Task(SEM_R2);
	if (Estat == LEC) ret = Sensor_Menu(PortNum);
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Telnet", (char *)"ENTREM COMUNICACIO PONT A Weather V3");
#endif
	Print_T_CRLF(PortTerm, (char *)"\r\n**********************************************");
	Print_T_CRLF(PortTerm, (char *)"* HELLO ...  INICI SESSIO REMOTA WEATHER V3  *");
	Print_T_CRLF(PortTerm, (char *)"**********************************************");
	Print_Text(PortTerm, (char *)Bufsen);
	Sensor_MenuRead();
	while ( !bucle )
	{
		if (Check_Read(PortTerm, (char *)Bufsen, (char *)"\r\n", 100, TIPSIZE) == TRUE)
		{
			string = strtok(Bufsen, (char *)"\r\n");
			if (strlen(string) == 1 && pos == 0)
			{
				strcpy(buf, string);
				opt = string[0];
				switch(opt)
				{
				  case 'X':
				  case 'x':
				  case 'S':
				  case 's':
				  default:
					  bucle = 1;
					  ret = Sensor_Set_Check(PortNum, buf, (char *)",*", 3000);
					  Sensor_Parse();
					  break;
				  case '1':
				  case '2':
				  case '4':
				  case '5':
				  case '9':
					  ret = Sensor_Set_Check(PortNum, buf, (char *)"2.", 3000);
					  break;
				  case '8':
					  ret = Sensor_Set_Check(PortNum, buf, (char *)"S.", 4000);
					  break;
				  case '3':
				  case '6':
				  case '7':
					  pos = 1;
					  if (opt == '3')	tip = 1;
					  ret = Sensor_Set_Check(PortNum, buf, (char *)":", 3000);
					  break;
				}
				Print_Text(PortTerm, (char *)Bufsen);
			}
			else if (strlen(string) > 0 && pos == 1)
			{
				pos = 0;
				sprintf(buf,"%s\n", string);
				if (tip == 1)	tip = 0; Segons = atoi(string);
				ret = Sensor_Set_Check(PortNum, buf, (char *)"S.", 4000);
				Print_Text(PortTerm, (char *)Bufsen);
				Sensor_MenuRead();
			}
			Bufsen[0]='\0';
		}
	}
	Print_T_CRLF(PortTerm, (char *)"**********************************************");
	Print_T_CRLF(PortTerm, (char *)"*    BYE... FI SESSIO REMOTA WEATHER V3      *");
	Print_T_CRLF(PortTerm, (char *)"**********************************************\r\n");
	Estat = LEC;
#if LOG >= LOG_INFO
	Log_T_Print(LOG_INFO, ret, (char *)"Sensor_Telnet", (char *)"SORTIM COMUNICACIO PONT A Weather V3");
#endif
	UnLock_Task(SEM_R2);
	free(buf);
	return ret;
}
