﻿/*
 * sleep.c
 *
 *  Created on: 11/05/2013
 *      Author: Jordi Fenés
 */

/***********************************************************************//**
 * @file		sleep.c -> based to AN10915 and derived of lpc17xx_clkpwr.c
 * @brief		Contains all functions support for Clock and Power Control
 * 				firmware library on LPC17xx
 * @version		3.0
 * @date		18. June. 2010
 * @author		NXP MCU SW Application Team
 **************************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "LPC17xx.h"
#include "sleep.h"

/* Public Functions ----------------------------------------------------------- */

/*********************************************************************//**
 * @brief 		Enter Sleep mode with co-operated instruction by the Cortex-M3.
 * @param[in]	None
 * @return		None
 **********************************************************************/
void Sleep(void)
{
	/*Clear the PCON*/
	LPC_SC->PCON = 0x0;
	/*Clear the SLEEPDEEP bit in the Cortex M3 System Control Register*/
	SCB->SCR = 0x00;
	/* Sleep Mode*/
	__WFI();
}


/*********************************************************************//**
 * @brief 		Enter Deep Sleep mode with co-operated instruction by the Cortex-M3.
 * @param[in]	None
 * @return		None
 **********************************************************************/
void DeepSleep(void)
{
    /* Deep-Sleep Mode, set SLEEPDEEP bit */
	SCB->SCR = 0x04;
	/* Normal mode of DEEPSLEEP */
	//LPC_SC->PCON = 0x0;
	/* Mode BODRPM */
	//LPC_SC->PCON = 0x4;
	/* Mode BOGD */
	LPC_SC->PCON = 0x8;
	/* Mode BORD */
	//LPC_SC->PCON = 0x10;
	/* Deep Sleep Mode*/
	__WFI();
}


/*********************************************************************//**
 * @brief 		Enter Power Down mode with co-operated instruction by the Cortex-M3.
 * @param[in]	None
 * @return		None
 **********************************************************************/
void PowerDown(void)
{
    /* Deep-Sleep Mode, set SLEEPDEEP bit */
	SCB->SCR = 0x04;
	LPC_SC->PCON = 0x9;
	/* Power Down Mode*/
	__WFI();
}


/*********************************************************************//**
 * @brief 		Enter Deep Power Down mode with co-operated instruction by the Cortex-M3.
 * @param[in]	None
 * @return		None
 **********************************************************************/
void DeepPowerDown(void)
{
    /* Deep-Sleep Mode, set SLEEPDEEP bit */
	SCB->SCR = 0x04;
	LPC_SC->PCON = 0x3;
	/* Deep Power Down Mode*/
	__WFI();
}

/* --------------------------------- End Of File ------------------------------ */

