/*
 * web.c
 *
 *  Created on: 15/05/2013
 *      Author: Administrador
 */

#include "type.h"
#include "logutil.h"
#include "wifly.h"
#include "sensor.h"
#include "web.h"
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//static char Receive[GETSIZE];

extern volatile LSensor Lectura;
static uint32_t Error_Send = 0, Error_Read = 0;

/*****************************************************************************
** Function name:		Web_Init
**
** Descriptions:		Defines the used UART, its baudrate and reset device
** 						This operation produce a few delay.
**
** parameters:			UART port ,  desired baudrate
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Web_Init(uint8_t PortNum, uint32_t Baudrate)
{
	uint8_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Task(SEM_R1);
	{
		ret = Wifly_Init(PortNum, Baudrate);
	}
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Web_End
**
** Descriptions:		Close the UART port used for WiFly device.
**
** parameters:			UART port
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Web_End(uint8_t PortNum)
{
	uint8_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Task(SEM_R1);
	{
		ret = Wifly_End(PortNum);
	}
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Web_Web_Send
**
** Descriptions:		Connect to specified Host and parameters (CGI address) to send LSensor structure
** 						with Mode HTTP/1.1
**
** parameters:			UART port to send, HOST to connect, sub-address in Host, Value to add
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint8_t Web_Send(uint8_t PortNum, char *Host, char *Port, char *Param)
{
	uint8_t ret = FALSE;
	char bufHost[BUFSIZE];
	char bufParam[GETSIZE];
	char **Receive = (char**)malloc(sizeof(char*));
	*Receive = NULL;
	Lock_Task(SEM_R1);
	sprintf(bufHost, "%s %s", Host, Port);

	sprintf(bufParam, "GET %s?Tem=%.2f&Hum=%.2f&Dew=%.2f", Param, Lectura.L_Tem, Lectura.L_Hum, Lectura.L_Dew);
	sprintf(bufParam, "%s&Bar=%.2f&Luz=%.2f&Ws=%.2f", bufParam, Lectura.L_Bar, Lectura.L_Luz, Lectura.L_Ws);
	sprintf(bufParam, "%s&Wd=%.2f&Rain=%.2f&Bat=%.2f", bufParam, Lectura.L_Wd, Lectura.L_Rain, Lectura.L_Bat);
	sprintf(bufParam, "%s %s\r\nhost: %s\r\n\r\n", bufParam, (char *)WEB, Host);

	ret = Wifly_Web(PortNum, bufHost, bufParam, Receive);
	if (ret == TRUE)
	{
		strcpy(bufParam, *Receive);
		Error_Send = 0;
	}
	else	Error_Send++;
#if LOG >= LOG_INFO
	Log_TT_Print(LOG_INFO, ret, (char *)"Web_Send", (char *)"ENVIO DADES INTERNET -> HTTP GET", Host);
#endif
	if (strstr(bufParam, (char *)"*CLOS*") == NULL)
	{
		if (Wifly_Set_Check(PortNum, (char *)"", (char *)"*CLOS*", 100) == FALSE)	Wifly_Close(PortNum);
	}
	Wifly_Exit(PortNum);
	UnLock_Task(SEM_R1);
	free(bufHost);
	free(bufParam);
	return ret;
}

/*****************************************************************************
** Function name:		Web_Web_Read
**
** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.1
** 						and collect received Value.
**
** parameters:			UART port to send, HOST to connect, sub-address in Host
** Returned value:		Value received or 0 if not correct
**
*****************************************************************************/
int32_t Web_Read(uint8_t PortNum, char *Host, char *Param)
{
	int32_t ret = 0;
	char bufHost[BUFSIZE];
	char bufParam[GETSIZE];
	char **Receive = (char**)malloc(sizeof(char*));
	*Receive = NULL;
	char *parse;
	Lock_Task(SEM_R1);
	sprintf(bufHost, "%s %s", Host, (char *)NETPORT);
	sprintf(bufParam, "GET %s %s\r\nhost: %s\r\n\r\n", Param, (char *)WEB, Host);
	if (Wifly_Web(PortNum, bufHost, bufParam, Receive) == TRUE)
	{
		strcpy(bufParam, *Receive);
		parse = strstr(bufParam, (char *)"200 OK");
		if  (parse == NULL)
		{
			ret = -1;
		}
		Error_Read = 0;
	}
	else	Error_Read++;
#if LOG >= LOG_INFO
	Log_TT_Print(LOG_INFO, ret, (char *)"Web_Read", (char *)"PETICIO INTERNET -> HTTP GET", Host);
#endif
	if (strstr(bufParam, (char *)"*CLOS*") == NULL)		Wifly_Close(PortNum);
	Wifly_Exit(PortNum);
	UnLock_Task(SEM_R1);
	free(bufHost);
	free(bufParam);
	return ret;
}

/*****************************************************************************
** Function name:		Web_Send_Error
**
** Descriptions:		Check the number of errors in send function and Initialize value.
**
** parameters:			Limit to check and True o false for initialize
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint32_t Web_Send_Error(uint32_t Init)
{
	uint32_t ret = Error_Send;
	Lock_Task(SEM_R1);
	{
		if(Init == TRUE)	Error_Send = 0;
	}
	UnLock_Task(SEM_R1);
	return ret;
}

/*****************************************************************************
** Function name:		Web_Read_Error
**
** Descriptions:		Check the number of errors in read function and Initialize value.
**
** parameters:			Limit to check and True o False for initialize
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint32_t Web_Read_Error(uint32_t Init)
{
	uint32_t ret = Error_Read;
	Lock_Task(SEM_R1);
	{
		if(Init == TRUE)	Error_Read = 0;
	}
	UnLock_Task(SEM_R1);
	return ret;
}
